#!/usr/bin/env python

"""
A script to make the sensitivity curve plots - include a joint LIGO curve S6, a joint VSR2/4 curve, and the LIGO S5
curve, and LIGO S3/S4 curve
"""

import sys
import math
import os
import numpy as np
import struct
import re

import matplotlib
from matplotlib import pyplot as plt
from scipy.interpolate import interp1d

# output the harmonic mean of the input spectra (where the input is a list of numpy arrays of time weighted
# power spectra)
def hmean_spec(wasds):
  for i, v in enumerate(wasds):
    if i == 0:
      mspec = np.divide(1., v)
    else:
      mspec = np.add(mspec, np.divide(1., v))
      
  return np.sqrt( np.divide( 1., mspec ) )

# load in the LIGO S6 amplitude spectral density curves
# taken from Latest 2010.06.07 S6 versions http://www.ligo.caltech.edu/~jzweizig/distribution/LSC_Data/
s6H1asd = np.loadtxt('lho4k_15May2010_05hrs17min45secUTC_strain.txt', comments='%')
s6L1asd = np.loadtxt('llo4k_31May2010_09hrs05min45secUTC_strain.txt', comments='%')

# load in the Virgo VSR2/4 amplitude spectral density curves
vsr2asd = np.loadtxt('SensitivityH_VSR2_091020.txt', comments='%')
vsr4asd = np.loadtxt('Sensitivity_VSR4_20110805_11d8Mpc.txt', comments='%')

# get a range of frequencies
fs = np.arange(10.1, 1500, 0.1)

# interpolate amplitude spectral densities to same freq range and convert to power spectra
intf = interp1d(s6H1asd[:,0], s6H1asd[:,1], kind='linear')
s6H1int = np.square(intf(fs))

intf = interp1d(s6L1asd[:,0], s6L1asd[:,1], kind='linear')
s6L1int = np.square(intf(fs))

intf = interp1d(vsr2asd[:,0], vsr2asd[:,1], kind='linear')
vsr2int = np.square(intf(fs))

intf = interp1d(vsr4asd[:,0], vsr4asd[:,1], kind='linear')
vsr4int = np.square(intf(fs))

# length of runs in seconds
TS6H1 = 238.*86400.
TS6L1 = 225.*86400.
TVSR2 = 149.*86400.
TVSR4 = 76.*86400.

# get S5 LIGO amplitude spectral densities
# taken from May 2007 (2007.05.25) versions http://www.ligo.caltech.edu/~jzweizig/distribution/LSC_Data/
s5H1asd = np.loadtxt('lho4k_070318_strain.txt', comments='%')
s5H2asd = np.loadtxt('lho2k_070514_strain.txt', comments='%')
s5L1asd = np.loadtxt('llo_060604_strain.txt', comments='%')

# interpolate amplitude spectral densities to same freq range and convert to power spectra
intf = interp1d(s5H1asd[:,0], s5H1asd[:,1], kind='linear')
s5H1int = np.square(intf(fs))

intf = interp1d(s5H2asd[:,0], s5H2asd[:,1], kind='linear')
s5H2int = np.square(intf(fs))

intf = interp1d(s5L1asd[:,0], s5L1asd[:,1], kind='linear')
s5L1int = np.square(intf(fs))

TS5H1 = 527.*86400.
TS5H2 = 535.*86400.
TS5L1 = 405.*86400.

# get S4 LIGO amplitude spectral densities
s4H1asd = np.loadtxt('lho4k_050226_strain.txt', comments='%')
s4H2asd = np.loadtxt('lho2k_050226_strain.txt', comments='%')
s4L1asd = np.loadtxt('llo_050311_strain.txt', comments='%')

# interpolate amplitude spectral densities to same freq range and convert to power spectra
intf = interp1d(s4H1asd[:,0], s4H1asd[:,1], kind='linear')
s4H1int = np.square(intf(fs))

intf = interp1d(s4H2asd[:,0], s4H2asd[:,1], kind='linear')
s4H2int = np.square(intf(fs))

intf = interp1d(s4L1asd[:,0], s4L1asd[:,1], kind='linear')
s4L1int = np.square(intf(fs))

TS4H1 = 19.4*86400.
TS4H2 = 22.5*86400.
TS4L1 = 17.1*86400.

# get S3 LIGO amplitude spectral densities
s3H1asd = np.loadtxt('lho4k_040104_strain.txt', comments='%')
s3H2asd = np.loadtxt('lho2k_031130_strain.txt', comments='%')
s3L1asd = np.loadtxt('llo_031227_strain.txt', comments='%')

# interpolate amplitude spectral densities to same freq range and convert to power spectra
intf = interp1d(s3H1asd[:,0], s3H1asd[:,1], kind='linear')
s3H1int = np.square(intf(fs))

intf = interp1d(s3H2asd[:,0], s3H2asd[:,1], kind='linear')
s3H2int = np.square(intf(fs))

intf = interp1d(s3L1asd[:,0], s3L1asd[:,1], kind='linear')
s3L1int = np.square(intf(fs))

TS3H1 = 45.5*86400.
TS3H2 = 42.1*86400.
TS3L1 = 13.4*86400.

# scale factor for sensitivity estimate
sf = 10.8 # value for the Bayesian time domain search 95% upper limit

# get the harmonic mean of the S3/S4 time weighted sensitivities
wsensS3S4 = [np.divide( s3H1int, TS3H1 ), np.divide( s3H2int, TS3H2 ), np.divide( s3L1int, TS3L1 ), np.divide( s4H1int, TS4H1 ), np.divide( s4H2int, TS4H2 ), np.divide( s4L1int, TS4L1 )]
S3S4sens = np.multiply( sf, hmean_spec(wsensS3S4) )

# get the harmonic mean of the S5 time weighted sensitivities
wsensS5 = [np.divide( s5H1int, TS5H1 ), np.divide( s5H2int, TS5H2 ), np.divide( s5L1int, TS5L1 )]
S5sens = np.multiply( sf, hmean_spec(wsensS5) )

# get the harmonic mean of the S6 time weighted sensitivities
wsensS6 = [np.divide( s6H1int, TS6H1 ), np.divide( s6L1int, TS6L1 )]
S6sens = np.multiply( sf, hmean_spec(wsensS6) )

wsensVSR = [np.divide( vsr2int, TVSR2 ), np.divide( vsr4int, TVSR4 )]
VSRsens = np.multiply(sf, hmean_spec(wsensVSR) )

# get advanced LIGO curve 
aLIGO = np.loadtxt('aligo_sensitivity.txt', comments='#')

# use one year of integration with TWO equally sensitive detectors at design sensitivity
TaLIGO = 365.25*86400.

# get advanced Virgo curve
aVirgo = np.loadtxt('adv_sensitivity.txt', comments='#')

# use one year of integration with detector at design sensitivity
TaVirgo = 365.25*86400.

# interpolate amplitude spectral densities to same freq range and convert to power spectra
intf = interp1d(aLIGO[:,0], aLIGO[:,4], kind='linear')
aLIGOpow = np.square(intf(fs))

intf = interp1d(aVirgo[:,0], aVirgo[:,6], kind='linear')
aVirgopow = np.square(intf(fs))

advsens = [np.divide( aLIGOpow, TaLIGO ), np.divide( aLIGOpow, TaLIGO ), np.divide( aVirgopow, TaVirgo )]
advsens = np.multiply( sf, hmean_spec(advsens) )

# load spin-down limits
sdlims = np.genfromtxt('spindownlims.txt', comments='#', dtype=None)

# load real upper limits
olims = np.genfromtxt('upperlims.txt', comments='#', dtype=None)
 
# plot the curves
mplparams = {
      'text.usetex': True, # use LaTeX for all text
      'axes.linewidth': 1, # set axes linewidths to 0.5
      'axes.grid': True, # add a grid
      'axes.axisbelow': True,
      'grid.linestyle': '-', # grid as solid lines not dashed
      'grid.color': 'lightgray', # grid lines grey
      'grid.linewidth': 0.5,
      'font.family': 'serif',
      'font.size': 18 }

matplotlib.rcParams.update(mplparams)

fig = plt.figure(figsize=(14,10),dpi=300)

plt.loglog(fs, S3S4sens, 'c', fs, VSRsens, 'b', fs, S5sens, 'm', fs, S6sens, 'g', linewidth=2)

plt.hold(True)

# plot advanced detectors curves
plt.loglog(fs, advsens, 'k', linewidth=2)

# pulsars with upper limits using Virgo only data
virgoonly = ['J0106+4855', 'J0609+2130', 'J1528-3146', 'J1718-3825', 'J1747-2958', 'J1748-2446J', 'J1753-1914', \
             'J1753-2240', 'J1809-1917', 'J1828-1101', 'J1831-0952', 'J1833-0827', 'J1856+0245', 'J1904+0412', \
             'J1915+1606', 'J1928+1746', 'J1954+2836', 'J2043+2740', 'J2235+1506']

# "interesting" pulsars
interesting = ['J0534+2200', 'J0537-6910', 'J0835-4510', 'J1813-1246', 'J1833-1034', 'J1913+1011', 'J1952+3252']

# pulsars with S3/S4 upper limits
s3s4pulsars = ['J1911+0101A', 'J1911+0101B', 'J1757-5322', 'J1629-6902', 'J1435-6100']

# pulsars with S5 upper limits
s5pulsars = ['J1701-3006B', 'J1701-3006C', 'J1748-2446ad', 'J1824-2452B', 'J1824-2452C', 'J1824-2452E', \
             'J1824-2452F', 'J1824-2452G', 'J1824-2452H', 'J1824-2452J', 'J1824-2452I']

# plot spin-down limits
for row in sdlims:
  plt.loglog(row[2], row[3], 'kv')

# plot upper limits
for row in olims:
  # plot interesting pulsars in yellow
  if row[1] in interesting:
    plt.loglog(row[2], row[3], '*', markerfacecolor='yellow', markersize=22, markeredgecolor='k')
  # plot s3s4 pulsars in grey
  elif row[1] in s3s4pulsars or row[1] in s5pulsars:
    plt.loglog(row[2], row[3], '*', markerfacecolor='lightgray', markersize=14, markeredgecolor='k')
  else:
    # otherwise just plot upper limit in red
    plt.loglog(row[2], row[3], 'r*', markersize=14, markeredgecolor='k')
  

plt.xlim(10, 1500)
plt.ylim(1e-27, 1e-23)
plt.xlabel(r'Gravitational-wave Frequency (Hz)', fontsize=20, fontweight=100)
plt.ylabel(r'Strain Sensitivity', fontsize=20, fontweight=100)

plt.legend(['S3/S4', 'VSR2/4', 'S5', 'S6', 'AdV/aLIGO'])
plt.setp(plt.gca().get_legend().get_texts(), fontsize='12')

plt.hold(False)

fig.subplots_adjust(left=0.18, bottom=0.15)

fig.savefig('senscurve.pdf')

mplparams = {
      'text.usetex': True, # use LaTeX for all text
      'axes.linewidth': 1, # set axes linewidths to 0.5
      'axes.grid': True, # add a grid
      'axes.axisbelow': True,
      'grid.linestyle': '-', # grid as solid lines not dashed
      'grid.color': 'lightgray', # grid lines grey
      'grid.linewidth': 0.5,
      'font.family': 'serif',
      'font.size': 32 }

matplotlib.rcParams.update(mplparams)
# create a zoomed version with just the interesting pulsars
fig = plt.figure(figsize=(14,10),dpi=300)

plt.loglog(fs, VSRsens, 'b', fs, S6sens, 'g', linewidth=4)

plt.hold(True)

# plot advanced detectors curves
plt.loglog(fs, advsens, 'k', linewidth=2)

# plot spin-down limits
for row in sdlims:
  if row[1] in interesting:
    plt.loglog(row[2], row[3], 'kv', markersize=24)

# plot upper limits
for row in olims:
  # plot interesting pulsars in yellow
  if row[1] in interesting:
    plt.loglog(row[2], row[3], '*', markerfacecolor='yellow', markersize=32, markeredgecolor='k')

plt.xlim(20, 150)
plt.ylim(1e-26, 5e-24)
plt.xlabel(r'Gravitational-wave Frequency (Hz)', fontsize=36, fontweight=100)
plt.ylabel(r'Strain Sensitivity', fontsize=36, fontweight=100)

plt.hold(False)

fig.subplots_adjust(left=0.18, bottom=0.15)

fig.savefig('senscurve_zoom.pdf')

